using System.Collections.Generic;
using UnityEditor.Experimental.GraphView;
using UnityEditor.MemoryProfiler;
using UnityEngine;
using UnityEngine.UIElements;

namespace ImageProcessingGraph.Editor.Unity_Image_Processing.Scripts.Editor.Windows
{

    public class ImageProcessingGraphEdgeConnectorListener : IEdgeConnectorListener
    {
        private GraphViewChange m_GraphViewChange;
        private List<Edge> m_EdgesToCreate;
        private List<GraphElement> m_EdgesToDelete;

        private ImageProcessingGraphViewWindow window;

        public ImageProcessingGraphEdgeConnectorListener(ImageProcessingGraphViewWindow window)
        {
            this.m_EdgesToCreate = new List<Edge>();
            this.m_EdgesToDelete = new List<GraphElement>();
            this.m_GraphViewChange.edgesToCreate = this.m_EdgesToCreate;
            this.window = window;
        }

        public void OnDropOutsidePort(Edge edge, Vector2 position)
        {
            window.searchProvider.target = (VisualElement)window.focusController.focusedElement;
            SearchWindow.Open(new SearchWindowContext(position), window.searchProvider);
            
            List<GraphConnection> connections = new List<GraphConnection>();
            
            foreach (var conn in window.asset.Connections)
            {
                if (conn.internalEdge == edge)
                {
                    connections.Add(conn);
                }
            }

            foreach (var VARIABLE in connections)
            {
                window.asset.Connections.Remove(VARIABLE);
            }
            
            ((ImageProcessingGraphNodeVisual)edge.input.node).ToggleExposedVariable((IPTPort)edge.input, true);
        }
        


        public void OnDrop(UnityEditor.Experimental.GraphView.GraphView graphView, Edge edge)
        {
            List<GraphConnection> connections = new List<GraphConnection>();
            
            foreach (var conn in window.asset.Connections)
            {
                if (conn.internalEdge == edge)
                {
                    connections.Add(conn);
                }
            }
            
            foreach (var VARIABLE in connections)
            {
                window.asset.Connections.Remove(VARIABLE);
            }
            
            ((ImageProcessingGraphNodeVisual)edge.input.node).ToggleExposedVariable((IPTPort)edge.input, true);
            
            
            this.m_EdgesToCreate.Clear();
            this.m_EdgesToCreate.Add(edge);
            this.m_EdgesToDelete.Clear();
            if (edge.input.capacity == Port.Capacity.Single)
            {
                foreach (Edge connection in edge.input.connections)
                {
                    if (connection != edge)
                        this.m_EdgesToDelete.Add((GraphElement)connection);
                }
            }

            if (edge.output.capacity == Port.Capacity.Single)
            {
                foreach (Edge connection in edge.output.connections)
                {
                    if (connection != edge)
                        this.m_EdgesToDelete.Add((GraphElement)connection);
                }
            }

            if (this.m_EdgesToDelete.Count > 0)
                graphView.DeleteElements((IEnumerable<GraphElement>)this.m_EdgesToDelete);
            List<Edge> edgesToCreate = this.m_EdgesToCreate;
            if (graphView.graphViewChanged != null)
                edgesToCreate = graphView.graphViewChanged(this.m_GraphViewChange).edgesToCreate;
            foreach (Edge edge1 in edgesToCreate)
            {
                graphView.AddElement((GraphElement)edge1);
                edge.input.Connect(edge1);
                edge.output.Connect(edge1);
            }
        }
    }
}