﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using ImageProcessingGraph.Editor.Nodes.NodeAttributes;
using UnityEditor;
using UnityEditor.Experimental.GraphView;
using UnityEditor.UIElements;
using UnityEngine;
using UnityEngine.UIElements;
using Input = ImageProcessingGraph.Editor.Nodes.NodeAttributes.Input;

namespace ImageProcessingGraph.Editor.Unity_Image_Processing.Scripts.Editor.Windows
{
    public class ImageProcessingGraphNodeVisual : Node
    {
        private BaseImageNode graphNode;
        public BaseImageNode GraphNode => graphNode;

        public List<Port> InputPorts { get; }
        public List<Port> OutputPorts { get; }

        private ImageProcessingGraphViewWindow window;
        
        private StyleSheet defaaStyleSheet;
        private StyleSheet errorStyleSheet;

        public ImageProcessingGraphNodeVisual(BaseImageNode node, ImageProcessingGraphViewWindow window)
        {
            this.AddToClassList("image-node-visual");
            this.window = window;

            graphNode = node;
            Type typeInfo = node.GetType();

            NodeInfoAttribute info = typeInfo.GetCustomAttribute<NodeInfoAttribute>();
            title = info.Title;

            string[] depths = info.MenuItem.Split('/');
            foreach (var depth in depths)
            {
                this.AddToClassList(depth.ToLower().Replace(' ', '-'));
            }

            this.InputPorts = new List<Port>();
            this.OutputPorts = new List<Port>();

            List<Input> inputs = new List<Input>();
            List<FieldInfo> inputFieldInfo = new List<FieldInfo>();
            List<FieldInfo> outputFieldInfo = new List<FieldInfo>();

            FieldInfo[] fields = typeInfo.GetFields(BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public);
            foreach (var field in fields)
            {
                if (field.GetCustomAttribute(typeof(Input)) != null)
                {
                    Input input = field.GetCustomAttribute<Input>();
                    inputs.Add(input);
                    inputFieldInfo.Add(field);
                }

                if (field.GetCustomAttribute(typeof(Output)) != null)
                {
                    Output output = field.GetCustomAttribute<Output>();
                    outputFieldInfo.Add(field);
                }
                    
            }
            
            CreateInputPorts(inputFieldInfo);
            CreateOutputPorts(outputFieldInfo);

            foreach (Port input in InputPorts)
            {
                
            }
            
            foreach (Port input in OutputPorts)
            {
                
            }
            
            defaaStyleSheet = AssetDatabase.LoadAssetAtPath<StyleSheet>("Assets/Unity Image Processing/Node.uss");
            if (defaaStyleSheet == null)
            {
                defaaStyleSheet = EditorGUIUtility.Load("Packages/com.chromium.imageprocessingrah/Node.uss") as StyleSheet;
            }
            styleSheets.Add(defaaStyleSheet);
            
            errorStyleSheet = AssetDatabase.LoadAssetAtPath<StyleSheet>("Assets/Unity Image Processing/NodeError.uss");
            if (errorStyleSheet == null)
            {
                errorStyleSheet = EditorGUIUtility.Load("Packages/com.chromium.imageprocessingrah/NodeError.uss") as StyleSheet;
            }

            graphNode.onFailed += () =>
            {
                styleSheets.Add(errorStyleSheet);
            };
            
            window.asset.OnRun += () =>
            {
                if (styleSheets.Contains(errorStyleSheet))
                    styleSheets.Remove(errorStyleSheet);
            };
            

            this.name = typeInfo.Name;
        }

        private void CreateInputPorts(List<FieldInfo> fields)
        {
            for (var index = 0; index < fields.Count; index++)
            {
                var field = fields[index];
                var port = IPTPort.Create(window.edgeConnectorListener, true, field.FieldType);
                
                string label = field.GetCustomAttribute<Input>().Label;
                if (label != "")
                    port.portName = label; 
                InputPorts.Add(port);

                inputContainer.Add(port);
                ExposeVariableToPort(port, field);
                port.fieldInfo = field;

            }
        }

        private void CreateOutputPorts(List<FieldInfo> fields)
        {
            for (var index = 0; index < fields.Count; index++)
            {
                var field = fields[index];
                var port = IPTPort.Create(window.edgeConnectorListener, false, field.FieldType);
                
                string label = field.GetCustomAttribute<Output>().Label;
                if (label != "")
                    port.portName = label; 
                OutputPorts.Add(port);
                outputContainer.Add(port);
                
                port.fieldInfo = field;
            }
        }

        // Exposes a variable on the port for editing when it's not connected
        public void ExposeVariableToPort(Port port, FieldInfo field)
        {
            VisualElement NewElement = new VisualElement();
            var ExposedPropertyContainer = ((IPTPort)port).ExposedPropertyContainer;
            Type containerType = null;
            
            if (ExposedPropertyContainer == null)
            {
                NewElement.name = "property-field-container";
                VisualElement the = CreatePropertyFieldForType(field.FieldType, field.GetValue(graphNode));
                
                if(the != null)
                    containerType = the.GetType();
                
                NewElement.Add(the);
                ((IPTPort)port).ExposedPropertyContainer = the;
                ExposedPropertyContainer = ((IPTPort)port).ExposedPropertyContainer;
            }
            else
            {
                containerType = ExposedPropertyContainer.GetType();
            }

            if (containerType == null)
                return;

            if (ExposedPropertyContainer.GetType() == typeof(IntegerField))
            {
                var intField = ExposedPropertyContainer as IntegerField;
                intField.RegisterValueChangedCallback(evt =>
                {
                    field.SetValue(graphNode, evt.newValue); // Update the field with the new value
                });
            }
            else if (ExposedPropertyContainer is FloatField floatField)
            {
                floatField.RegisterValueChangedCallback(evt =>
                {
                    field.SetValue(graphNode, evt.newValue); // Update the field with the new value
                });
            }
            else if (ExposedPropertyContainer is Toggle boolField)
            {
                boolField.RegisterValueChangedCallback(evt =>
                {
                    field.SetValue(graphNode, evt.newValue); // Update the field with the new value
                });
            }
            else if (ExposedPropertyContainer is TextField stringField)
            {
                stringField.RegisterValueChangedCallback(evt =>
                {
                    field.SetValue(graphNode, evt.newValue); // Update the field with the new value
                });
            }
            else if (ExposedPropertyContainer is ColorField colorField)
            {
                colorField.RegisterValueChangedCallback(evt =>
                {
                    field.SetValue(graphNode, evt.newValue); // Update the field with the new value
                });
            }
            else if (ExposedPropertyContainer is Vector3Field vector3Field)
            {
                vector3Field.RegisterValueChangedCallback(evt =>
                {
                    field.SetValue(graphNode, evt.newValue); // Update the field with the new value
                });
            }
            else if (ExposedPropertyContainer is Vector2Field vector2Field)
            {
                vector2Field.RegisterValueChangedCallback(evt =>
                {
                    field.SetValue(graphNode, evt.newValue); // Update the field with the new value
                });
            }
            else if (ExposedPropertyContainer is ObjectField objectField)
            {
                objectField.RegisterValueChangedCallback(evt =>
                {
                    field.SetValue(graphNode, evt.newValue); // Update the field with the new value
                });
            }
            else if (ExposedPropertyContainer is EnumField enumField)
            {
                enumField.RegisterValueChangedCallback(evt =>
                {
                    field.SetValue(graphNode, evt.newValue); // 🎯 Update the field with the new enum value
                });
            }
            else if (ExposedPropertyContainer.GetType() == typeof(GreyscaleField))
            {
                var greyscaleField = ExposedPropertyContainer as GreyscaleField;
                greyscaleField.RegisterValueChangedCallback(evt =>
                {
                    var value = (GreyscaleValue)field.GetValue(graphNode);

                    if (evt.newValue > greyscaleField.minMax.Item2)
                        value.value = greyscaleField.minMax.Item2;
                    else if (evt.newValue < greyscaleField.minMax.Item1) value.value = greyscaleField.minMax.Item1;

                    value.value = evt.newValue;
                });
            }
            
            port.Add(NewElement);
        }

        public void ToggleExposedVariable(Port port, bool value)
        {
            IPTPort iptPort = port as IPTPort;
            if(iptPort.ExposedPropertyContainer != null)
                iptPort.ExposedPropertyContainer.visible = value;
        }
        
        private VisualElement CreatePropertyFieldForType(Type type, object value)
        {
            if (type == typeof(int))
            {
                var intField = new IntegerField { value = (int)value };
                return intField;
            }
            else if (type == typeof(float))
            {
                var floatField = new FloatField { value = (float)value };
                return floatField;
            }
            else if (type == typeof(bool))
            {
                var boolField = new Toggle { value = (bool)value };
                return boolField;
            }
            else if (type == typeof(string))
            {
                var stringField = new TextField { value = (string)value };
                return stringField;
            }
            else if (type == typeof(Color))
            {
                var colorField = new ColorField() { value = (Color)value };
                return colorField;
            }
            else if (type == typeof(Vector3))
            {
                var vector3Field = new Vector3Field { value = (Vector3)value };
                return vector3Field;
            }
            else if (type == typeof(Vector2))
            {
                var vector2Field = new Vector2Field { value = (Vector2)value };
                return vector2Field;
            }
            else if (type == typeof(Texture2D))
            {
                var objectField = new ObjectField { value = (Texture2D)value, objectType = typeof(Texture2D) };
                return objectField;
            }
            else if (type.IsEnum) // 💥✨ ENUMS, BABY! 💥✨
            {
                var enumField = new EnumField((Enum)value);
                return enumField;
            }
            else if (type == typeof(GreyscaleValue))
            {

                var greyscaleValue = (GreyscaleValue)value;
                var intField = new GreyscaleField { value = (int)greyscaleValue.value };
                return intField;
            }

            // Add more types as needed
            return null;
        }


        public void SavePosition() => graphNode.SetPosition(GetPosition());
    }
}
